@@ -1587,18 +1587,13 @@ static const GtkSyntaxTag *
 get_syntax_start (GtkSourceBuffer      *source_buffer,
 		  const gchar          *text,
 		  gint                  length,
+		  guint                 match_options,
 		  GtkSourceBufferMatch *match)
 {
 	GList *list;
 	GtkSyntaxTag *tag;
 	gint pos;
 	
-	/*
-	g_return_val_if_fail (text != NULL, NULL);
-	g_return_val_if_fail (length >= 0, NULL);
-	g_return_val_if_fail (match != NULL, NULL);
-	*/
-
 	if (length == 0)
 		return NULL;
 	
@@ -1615,7 +1610,8 @@ get_syntax_start (GtkSourceBuffer      *source_buffer,
 			text,
 			pos,
 			length,
-			match);
+			match,
+			match_options);
 		if (pos < 0 || !is_escaped (source_buffer, text, match->startindex))
 			break;
 		pos = match->startpos + 1;
@@ -1628,7 +1624,8 @@ get_syntax_start (GtkSourceBuffer      *source_buffer,
 		tag = list->data;
 		
 		if (gtk_source_regex_match (tag->reg_start, text,
-					    pos, match->endindex))
+					    pos, match->endindex,
+					    match_options))
 			return tag;
 
 		list = g_list_next (list);
@@ -1641,6 +1638,7 @@ static gboolean
 get_syntax_end (GtkSourceBuffer      *source_buffer,
 		const gchar          *text,
 		gint                  length,
+		guint                 match_options,
 		GtkSyntaxTag         *tag,
 		GtkSourceBufferMatch *match)
 {
@@ -1657,7 +1655,7 @@ get_syntax_end (GtkSourceBuffer      *source_buffer,
 	pos = 0;
 	do {
 		pos = gtk_source_regex_search (tag->reg_end, text, pos,
-					       length, match);
+					       length, match, match_options);
 		if (pos < 0 || !is_escaped (source_buffer, text, match->startindex))
 			break;
 		pos = match->startpos + 1;
@@ -1856,6 +1854,7 @@ next_syntax_region (GtkSourceBuffer      *source_buffer,
 		    const gchar          *head,
 		    gint                  head_length,
 		    gint                  head_offset,
+		    guint                 head_options,
 		    GtkSourceBufferMatch *match)
 {
 	GtkSyntaxTag *tag;
@@ -1865,7 +1864,7 @@ next_syntax_region (GtkSourceBuffer      *source_buffer,
 		/* we come from a non-syntax colored region, so seek
 		 * for an opening pattern */
 		tag = (GtkSyntaxTag *) get_syntax_start (
-			source_buffer, head, head_length, match);
+			source_buffer, head, head_length, head_options, match);
 
 		if (!tag)
 			return FALSE;
@@ -1878,7 +1877,7 @@ next_syntax_region (GtkSourceBuffer      *source_buffer,
 		/* seek the closing pattern for the current syntax
 		 * region */
 		found = get_syntax_end (source_buffer,
-					head, head_length,
+					head, head_length, head_options, 
 					state->tag, match);
 		
 		if (!found)
@@ -1903,6 +1902,7 @@ build_syntax_regions_table (GtkSourceBuffer   *source_buffer,
 	gboolean use_old_data;
 	gchar *slice, *head;
 	gint offset, head_length;
+	guint slice_options;
 	GtkSourceBufferMatch match;
 	SyntaxDelimiter delim;
 	GTimer *timer;
@@ -1962,6 +1962,10 @@ build_syntax_regions_table (GtkSourceBuffer   *source_buffer,
 	head = slice;
 	head_length = strlen (head);
 
+	/* we always stop processing at line ends */
+	slice_options = (gtk_text_iter_get_line_offset (&start) != 0 ?
+			 GTK_SOURCE_REGEX_NOT_BOL : 0);
+
 	timer = g_timer_new ();
 
 	/* MAIN LOOP: build the table */
@@ -1971,6 +1975,7 @@ build_syntax_regions_table (GtkSourceBuffer   *source_buffer,
 					 head,
 					 head_length,
 					 offset,
+					 slice_options,
 					 &match)) {
 			/* no further data */
 			break;
@@ -2004,6 +2009,18 @@ build_syntax_regions_table (GtkSourceBuffer   *source_buffer,
 		head += match.endindex;
 		head_length -= match.endindex;
 		offset += match.endpos;
+
+		/* recalculate b-o-l matching options */
+		if (match.endindex > 0)
+		{
+			GtkTextIter tmp;
+			gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (source_buffer),
+							    &tmp, offset);
+			if (gtk_text_iter_get_line_offset (&tmp) != 0)
+				slice_options |= GTK_SOURCE_REGEX_NOT_BOL;
+			else
+				slice_options &= ~GTK_SOURCE_REGEX_NOT_BOL;
+		}
 	}
     
 	g_free (slice);
@@ -2075,6 +2092,7 @@ update_syntax_regions (GtkSourceBuffer *source_buffer,
 	gint first_region, region;
 	gint table_index, expected_end_index;
 	gchar *slice, *head;
+	guint slice_options;
 	gint head_length, head_offset;
 	GtkTextIter start_iter, end_iter;
 	GtkSourceBufferMatch match;
@@ -2214,6 +2232,12 @@ update_syntax_regions (GtkSourceBuffer *source_buffer,
 	head = slice = gtk_text_iter_get_slice (&start_iter, &end_iter);
 	head_length = strlen (head);
 
+	/* eol match options is constant for this run */
+	slice_options = ((gtk_text_iter_get_line_offset (&start_iter) != 0 ?
+			  GTK_SOURCE_REGEX_NOT_BOL : 0) |
+			 (!gtk_text_iter_ends_line (&end_iter) ?
+			  GTK_SOURCE_REGEX_NOT_EOL : 0));
+
 	/* We will start analyzing the slice of text and see if it
 	 * matches the information from the table.  When we hit a
 	 * mismatch, it means we need to invalidate. */
@@ -2223,6 +2247,7 @@ update_syntax_regions (GtkSourceBuffer *source_buffer,
 				   head,
 				   head_length,
 				   head_offset,
+				   slice_options,
 				   &match)) {
 		/* correct offset, since the table has the old offsets */
 		if (delim.offset > start_offset + delta)
@@ -2245,6 +2270,18 @@ update_syntax_regions (GtkSourceBuffer *source_buffer,
 		head_length -= match.endindex;
 		head_offset += match.endpos;
 		table_index++;
+
+		/* recalculate b-o-l matching options */
+		if (match.endindex > 0)
+		{
+			GtkTextIter tmp;
+			gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (source_buffer),
+							    &tmp, head_offset);
+			if (gtk_text_iter_get_line_offset (&tmp) != 0)
+				slice_options |= GTK_SOURCE_REGEX_NOT_BOL;
+			else
+				slice_options &= ~GTK_SOURCE_REGEX_NOT_BOL;
+		}
 	}
 
 	g_free (slice);
@@ -2316,6 +2353,7 @@ search_patterns (GList       *matches,
 		 gint         length,
 		 gint         offset,
 		 gint         index,
+		 guint        match_options,
 		 GList       *patterns)
 {
 	GtkSourceBufferMatch match;
@@ -2351,7 +2389,8 @@ search_patterns (GList       *matches,
 					     text,
 					     0,
 					     length,
-					     &match);
+					     &match,
+					     match_options);
 		
 		if (i >= 0 && match.endpos != i) {
 			GList *p;
@@ -2404,7 +2443,8 @@ static void
 check_pattern (GtkSourceBuffer *source_buffer,
 	       GtkTextIter     *start,
 	       const gchar     *text,
-	       gint             length)
+	       gint             length,
+	       guint            match_options)
 {
 	GList *matches;
 	gint offset, index;
@@ -2437,6 +2477,7 @@ check_pattern (GtkSourceBuffer *source_buffer,
 	matches = search_patterns (NULL,
 				   ptr, length,
 				   offset, index,
+				   match_options,
 				   gtk_source_buffer_get_pattern_entries (source_buffer));
 	
 	while (matches && length > 0) {
@@ -2464,6 +2505,7 @@ check_pattern (GtkSourceBuffer *source_buffer,
 		matches = search_patterns (matches,
 					   ptr, length,
 					   offset, index,
+					   match_options,
 					   NULL);
 	}
 
@@ -2491,6 +2533,7 @@ highlight_region (GtkSourceBuffer *source_buffer,
 	GArray *table;
 	gint region;
 	gchar *slice, *slice_ptr;
+	guint slice_options;
 	GTimer *timer;
 
 	timer = NULL;
@@ -2557,8 +2600,16 @@ highlight_region (GtkSourceBuffer *source_buffer,
 			   non-syntax patterns */
 			tmp = g_utf8_offset_to_pointer (slice_ptr,
 							e_off - b_off);
+
+			/* calculate beginning and end of line match options */
+			slice_options = ((gtk_text_iter_get_line_offset (&b_iter) != 0 ?
+					  GTK_SOURCE_REGEX_NOT_BOL : 0) |
+					 (!gtk_text_iter_ends_line (&e_iter) ?
+					  GTK_SOURCE_REGEX_NOT_EOL : 0));
+
 			check_pattern (source_buffer, &b_iter,
-				       slice_ptr, tmp - slice_ptr);
+				       slice_ptr, tmp - slice_ptr,
+				       slice_options);
 			slice_ptr = tmp;
 		}
 
