@@ -189,7 +189,7 @@ enum
 	PROP_PRINT_NUMBERS,
 	PROP_PRINT_HEADER,
 	PROP_PRINT_FOOTER,
-	PROP_HEADER_FOOTER_FONT,
+	PROP_HEADER_FOOTER_FONT
 };
 
 enum
@@ -1157,7 +1157,7 @@ break_line (GtkSourcePrintJob *job,
 
 	set_style (job, segment->style);
 	
-	while (ch != '\n' && ch != '\r')
+	while (ch != 0 && ch != '\n' && ch != '\r')
 	{
 	       	gint glyph;
 		
